﻿namespace AirplaneWithRadar
{
	// Параметризованный набор объектов
	internal class SetAirplaneGeneric<T>
		where T : class
	{
		// Массив объектов, которые храним
		private readonly T[] _places;
		// Количество объектов в массиве
		public int Count => _places.Length;
		// Конструктор
		public SetAirplaneGeneric(int count)
		{
			_places = new T[count];
		}
		// Добавление объекта в набор
		public int Insert(T airplane)
		{
			// вставка в начало набора
			return Insert(airplane, 0);
		}
		// Добавление объекта в набор на конкретную позицию
		public int Insert(T airplane, int position)
		{
			//  проверка позиции
			if (position >= _places.Length || position < 0)
				return -1;
			//проверка, что элемент массива по этой позиции пустой, если нет, то
			if (_places[position] == null)
			{
				_places[position] = airplane;
				return position;
			}
			//проверка, что после вставляемого элемента в массиве есть пустой элемент
			int findEmptyPos = -1;

			for (int i = position + 1; i < Count; i++)
			{
				if (_places[i] == null)
				{
					findEmptyPos = i;
					break;
				}
			}
			if (findEmptyPos < 0) return -1;

			//сдвиг всех объектов, находящихся справа от позиции до первого пустого элемента
			for (int i = findEmptyPos; i > position; i--)
			{
				_places[i] = _places[i - 1];
			}
			// вставка по позиции
			_places[position] = airplane;
			return position;
		}
		// Удаление объекта из набора с конкретной позиции
		public T Remove(int position)
		{
			// проверка позиции
			if (position >= _places.Length || position < 0) return null;
			// удаление объекта из массива, присовив элементу массива значение null
			T temp = _places[position];
			_places[position] = null;
			return temp;
		}
		// Получение объекта из набора по позиции
		public T Get(int position)
		{
			// проверка позиции
			if (position >= _places.Length || position < 0)
				return null;
			return _places[position];
		}
	}
}